//
//  NisusCollection.h
//  NisusCore
//
//  Created by Martin on 2015.07.20.
//  Copyright (c) 2015 Nisus. All rights reserved.
//
// A protocol that Cocoa collections already implement, but we explicitly codify for APIs.

#import <Foundation/Foundation.h>



@protocol NisusCollection <NSObject, NSFastEnumeration>
- (NSUInteger) count;
- (NSArray*) allObjectsNisus ;
- (BOOL) containsObjectNisus:(id)obj ;			// uses value equality via -isEquals (not reference/pointer equality)
- (id) anyObjectNisus ;
@end



@protocol NisusMutableCollection <NisusCollection>
- (void) addObject:(id)obj ;
- (void) addObjectIfNotNilNisus:(id)obj ;
- (void) addObjectsFromNisusCollection:(id<NisusCollection>)addObjects ;
@end




// This class is never instantiated or subclassed, it's just to provide class methods for all collection types.
@interface NisusCollection : NSObject
+ (void) addObjects:(id<NisusCollection>)addObjects toCollection:(id<NisusMutableCollection>)collection filter:(id(NS_NOESCAPE^)(id object))filter ;
@end




// Cocoa class conformance.
@interface NSArray (NisusCollection) <NisusCollection>
+ (instancetype) arrayWithNisusCollection:(id<NisusCollection>)collection filter:(id(NS_NOESCAPE^)(id object))filter ;
@end

@interface NSMutableArray (NisusMutableCollection) <NisusMutableCollection>
@end



@interface NSSet (NisusCollection) <NisusCollection>
+ (instancetype) setWithNisusCollection:(id<NisusCollection>)collection filter:(id(NS_NOESCAPE^)(id object))filter ;
@end

@interface NSMutableSet (NisusMutableCollection) <NisusMutableCollection>
@end



// We consider a dictionary to be a collection of keys, not their mapped values.
@interface NSDictionary (NisusCollection) <NisusCollection>
@end
