//
//  NisusVersion.h
//  NisusCore
//
//  Created by Martin on 2018.11.09.
//  Copyright © 2018 Nisus. All rights reserved.
//
//	Helper class to parse and represent app versions.
//
//	Consider a build of Nisus Writer Pro 3.1.4 beta 7:
//		The version display string is "3.1.4 beta 7"
//		With version components: major=3, minor=1, rev=4, release type = beta (2), and release increment = 7
//		Bundle version string is "0301.0402.07"
//

#import <Foundation/Foundation.h>

typedef uint8_t NisusVersionComponent; // our components are from 1-99 since we only have two digits per component in the bundle version string
#define NisusVersionComponent_MAX (99)

typedef NS_ENUM( NSUInteger, NisusVersionReleaseType ){
	NisusVersionReleaseType_INVALID			= 0,
	NisusVersionReleaseType_Alpha			= 1,
	NisusVersionReleaseType_Beta			= 2,
	NisusVersionReleaseType_FinalCandidate	= 3,
	NisusVersionReleaseType_Public			= 4,
};
#define NisusVersionReleaseType_MIN_VALID (1)
#define NisusVersionReleaseType_MAX_VALID (4)



@interface NisusVersion : NSObject <NSCopying>

- (instancetype) initWithDisplayString:(NSString*)str error:(NSError**)errorPtr ;
- (instancetype) initWithBundleVersionString:(NSString*)str error:(NSError**)errorPtr ;
- (instancetype) initWithMajor:(NisusVersionComponent)major minor:(NisusVersionComponent)minor revision:(NisusVersionComponent)rev ;
- (instancetype) initWithMajor:(NisusVersionComponent)major minor:(NisusVersionComponent)minor revision:(NisusVersionComponent)rev releaseType:(NisusVersionReleaseType)relType releaseIncrement:(NisusVersionComponent)relInc ;

@property (readonly) NisusVersionComponent majorVersion ;
@property (readonly) NisusVersionComponent minorVersion ;
@property (readonly) NisusVersionComponent revision ;
@property (readonly) NisusVersionReleaseType releaseType ;
@property (readonly) NisusVersionComponent releaseIncrement ;

@property (readonly) NSString* displayString ;
@property (readonly) NSString* bundleVersionString ;

// comparisons
- (BOOL) isEqualVersion:(NisusVersion*)vers ;
- (NSComparisonResult) compareVersion:(NisusVersion*)vers ;

@end

